@@ -1,6 +1,6 @@
 package Yukki::Web::Router;
 BEGIN {
-  $Yukki::Web::Router::VERSION = '0.110880';
+  $Yukki::Web::Router::VERSION = '0.110900';
 }
 use Moose;
 
@@ -71,14 +71,14 @@ sub BUILD {
             repository => 'main',
         },
         validations => {
-            action     => qr/^(?:view|edit|preview|attach)$/,
+            action     => qr/^(?:view|edit|history|diff|preview|attach)$/,
             repository => qr/^[_a-z0-9]+$/i,
             page       => subtype('ArrayRef[Str]' => where {
                 all { /^[_a-z0-9-.]+(?:\.[_a-z0-9-]+)*$/i } @$_
             }),
         },
         acl => [
-            [ read  => { action => [ qw( view preview ) ] } ],
+            [ read  => { action => [ qw( view preview history diff ) ] } ],
             [ write => { action => [ qw( edit attach ) ]  } ],
         ],
         target => $self->controller('Page'),
@@ -116,7 +116,7 @@ Yukki::Web::Router - send requests to the correct controllers, yo
 
 =head1 VERSION
 
-version 0.110880
+version 0.110900
 
 =head1 DESCRIPTION
 
